# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api/stats/dropshot.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ...api import player_id_pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='api/stats/dropshot.proto',
  package='api.stats',
  serialized_pb=_b('\n\x18\x61pi/stats/dropshot.proto\x12\tapi.stats\x1a\x13\x61pi/player_id.proto\"?\n\x0c\x44ropshotTile\x12\n\n\x02id\x18\x01 \x01(\x05\x12#\n\x05state\x18\x02 \x01(\x0e\x32\x14.api.stats.TileState\":\n\x16\x44ropshotTileDamageStat\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x14\n\x0ctotal_damage\x18\x02 \x01(\x05\"|\n\x11\x44ropshotTileStats\x12\x37\n\x0c\x64\x61mage_stats\x18\x01 \x03(\x0b\x32!.api.stats.DropshotTileDamageStat\x12\x15\n\rdamaged_tiles\x18\x02 \x01(\x05\x12\x17\n\x0f\x64\x65stroyed_tiles\x18\x03 \x01(\x05\"@\n\rDropshotStats\x12\x14\n\x0ctotal_damage\x18\x01 \x01(\x05\x12\x19\n\x11\x64\x61mage_efficiency\x18\x02 \x01(\x02\"T\n\x16\x44ropshotBallPhaseStats\x12\r\n\x05phase\x18\x01 \x01(\x05\x12\x0f\n\x07\x61verage\x18\x02 \x01(\x02\x12\x0b\n\x03max\x18\x03 \x01(\x02\x12\r\n\x05total\x18\x04 \x01(\x02\"\x87\x01\n\x13\x44ropshotDamageEvent\x12 \n\tplayer_id\x18\x01 \x01(\x0b\x32\r.api.PlayerId\x12\x14\n\x0c\x66rame_number\x18\x02 \x01(\x05\x12\x10\n\x08tile_hit\x18\x03 \x01(\x05\x12&\n\x05tiles\x18\x04 \x03(\x0b\x32\x17.api.stats.DropshotTile\"j\n\x1c\x44ropshotBallPhaseChangeEvent\x12\x14\n\x0c\x66rame_number\x18\x01 \x01(\x05\x12\x12\n\nball_phase\x18\x02 \x01(\x05\x12 \n\tplayer_id\x18\x03 \x01(\x0b\x32\r.api.PlayerId\"\xc0\x01\n\x11\x44ropshotGameStats\x12\x35\n\rdamage_events\x18\x01 \x03(\x0b\x32\x1e.api.stats.DropshotDamageEvent\x12\x42\n\x11\x62\x61ll_phase_events\x18\x02 \x03(\x0b\x32\'.api.stats.DropshotBallPhaseChangeEvent\x12\x30\n\ntile_stats\x18\x03 \x01(\x0b\x32\x1c.api.stats.DropshotTileStats*6\n\tTileState\x12\r\n\tUNDAMAGED\x10\x00\x12\x0b\n\x07\x44\x41MAGED\x10\x01\x12\r\n\tDESTROYED\x10\x02')
  ,
  dependencies=[player_id_pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_TILESTATE = _descriptor.EnumDescriptor(
  name='TileState',
  full_name='api.stats.TileState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNDAMAGED', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DAMAGED', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DESTROYED', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=904,
  serialized_end=958,
)
_sym_db.RegisterEnumDescriptor(_TILESTATE)

TileState = enum_type_wrapper.EnumTypeWrapper(_TILESTATE)
UNDAMAGED = 0
DAMAGED = 1
DESTROYED = 2



_DROPSHOTTILE = _descriptor.Descriptor(
  name='DropshotTile',
  full_name='api.stats.DropshotTile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='api.stats.DropshotTile.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='state', full_name='api.stats.DropshotTile.state', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=123,
)


_DROPSHOTTILEDAMAGESTAT = _descriptor.Descriptor(
  name='DropshotTileDamageStat',
  full_name='api.stats.DropshotTileDamageStat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='api.stats.DropshotTileDamageStat.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total_damage', full_name='api.stats.DropshotTileDamageStat.total_damage', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=125,
  serialized_end=183,
)


_DROPSHOTTILESTATS = _descriptor.Descriptor(
  name='DropshotTileStats',
  full_name='api.stats.DropshotTileStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='damage_stats', full_name='api.stats.DropshotTileStats.damage_stats', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='damaged_tiles', full_name='api.stats.DropshotTileStats.damaged_tiles', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='destroyed_tiles', full_name='api.stats.DropshotTileStats.destroyed_tiles', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=185,
  serialized_end=309,
)


_DROPSHOTSTATS = _descriptor.Descriptor(
  name='DropshotStats',
  full_name='api.stats.DropshotStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='total_damage', full_name='api.stats.DropshotStats.total_damage', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='damage_efficiency', full_name='api.stats.DropshotStats.damage_efficiency', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=311,
  serialized_end=375,
)


_DROPSHOTBALLPHASESTATS = _descriptor.Descriptor(
  name='DropshotBallPhaseStats',
  full_name='api.stats.DropshotBallPhaseStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='phase', full_name='api.stats.DropshotBallPhaseStats.phase', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='average', full_name='api.stats.DropshotBallPhaseStats.average', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max', full_name='api.stats.DropshotBallPhaseStats.max', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='total', full_name='api.stats.DropshotBallPhaseStats.total', index=3,
      number=4, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=377,
  serialized_end=461,
)


_DROPSHOTDAMAGEEVENT = _descriptor.Descriptor(
  name='DropshotDamageEvent',
  full_name='api.stats.DropshotDamageEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='player_id', full_name='api.stats.DropshotDamageEvent.player_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='frame_number', full_name='api.stats.DropshotDamageEvent.frame_number', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tile_hit', full_name='api.stats.DropshotDamageEvent.tile_hit', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tiles', full_name='api.stats.DropshotDamageEvent.tiles', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=464,
  serialized_end=599,
)


_DROPSHOTBALLPHASECHANGEEVENT = _descriptor.Descriptor(
  name='DropshotBallPhaseChangeEvent',
  full_name='api.stats.DropshotBallPhaseChangeEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='frame_number', full_name='api.stats.DropshotBallPhaseChangeEvent.frame_number', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ball_phase', full_name='api.stats.DropshotBallPhaseChangeEvent.ball_phase', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='player_id', full_name='api.stats.DropshotBallPhaseChangeEvent.player_id', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=601,
  serialized_end=707,
)


_DROPSHOTGAMESTATS = _descriptor.Descriptor(
  name='DropshotGameStats',
  full_name='api.stats.DropshotGameStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='damage_events', full_name='api.stats.DropshotGameStats.damage_events', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='ball_phase_events', full_name='api.stats.DropshotGameStats.ball_phase_events', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='tile_stats', full_name='api.stats.DropshotGameStats.tile_stats', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=710,
  serialized_end=902,
)

_DROPSHOTTILE.fields_by_name['state'].enum_type = _TILESTATE
_DROPSHOTTILESTATS.fields_by_name['damage_stats'].message_type = _DROPSHOTTILEDAMAGESTAT
_DROPSHOTDAMAGEEVENT.fields_by_name['player_id'].message_type = player_id_pb2._PLAYERID
_DROPSHOTDAMAGEEVENT.fields_by_name['tiles'].message_type = _DROPSHOTTILE
_DROPSHOTBALLPHASECHANGEEVENT.fields_by_name['player_id'].message_type = player_id_pb2._PLAYERID
_DROPSHOTGAMESTATS.fields_by_name['damage_events'].message_type = _DROPSHOTDAMAGEEVENT
_DROPSHOTGAMESTATS.fields_by_name['ball_phase_events'].message_type = _DROPSHOTBALLPHASECHANGEEVENT
_DROPSHOTGAMESTATS.fields_by_name['tile_stats'].message_type = _DROPSHOTTILESTATS
DESCRIPTOR.message_types_by_name['DropshotTile'] = _DROPSHOTTILE
DESCRIPTOR.message_types_by_name['DropshotTileDamageStat'] = _DROPSHOTTILEDAMAGESTAT
DESCRIPTOR.message_types_by_name['DropshotTileStats'] = _DROPSHOTTILESTATS
DESCRIPTOR.message_types_by_name['DropshotStats'] = _DROPSHOTSTATS
DESCRIPTOR.message_types_by_name['DropshotBallPhaseStats'] = _DROPSHOTBALLPHASESTATS
DESCRIPTOR.message_types_by_name['DropshotDamageEvent'] = _DROPSHOTDAMAGEEVENT
DESCRIPTOR.message_types_by_name['DropshotBallPhaseChangeEvent'] = _DROPSHOTBALLPHASECHANGEEVENT
DESCRIPTOR.message_types_by_name['DropshotGameStats'] = _DROPSHOTGAMESTATS
DESCRIPTOR.enum_types_by_name['TileState'] = _TILESTATE

DropshotTile = _reflection.GeneratedProtocolMessageType('DropshotTile', (_message.Message,), dict(
  DESCRIPTOR = _DROPSHOTTILE,
  __module__ = 'api.stats.dropshot_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.DropshotTile)
  ))
_sym_db.RegisterMessage(DropshotTile)

DropshotTileDamageStat = _reflection.GeneratedProtocolMessageType('DropshotTileDamageStat', (_message.Message,), dict(
  DESCRIPTOR = _DROPSHOTTILEDAMAGESTAT,
  __module__ = 'api.stats.dropshot_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.DropshotTileDamageStat)
  ))
_sym_db.RegisterMessage(DropshotTileDamageStat)

DropshotTileStats = _reflection.GeneratedProtocolMessageType('DropshotTileStats', (_message.Message,), dict(
  DESCRIPTOR = _DROPSHOTTILESTATS,
  __module__ = 'api.stats.dropshot_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.DropshotTileStats)
  ))
_sym_db.RegisterMessage(DropshotTileStats)

DropshotStats = _reflection.GeneratedProtocolMessageType('DropshotStats', (_message.Message,), dict(
  DESCRIPTOR = _DROPSHOTSTATS,
  __module__ = 'api.stats.dropshot_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.DropshotStats)
  ))
_sym_db.RegisterMessage(DropshotStats)

DropshotBallPhaseStats = _reflection.GeneratedProtocolMessageType('DropshotBallPhaseStats', (_message.Message,), dict(
  DESCRIPTOR = _DROPSHOTBALLPHASESTATS,
  __module__ = 'api.stats.dropshot_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.DropshotBallPhaseStats)
  ))
_sym_db.RegisterMessage(DropshotBallPhaseStats)

DropshotDamageEvent = _reflection.GeneratedProtocolMessageType('DropshotDamageEvent', (_message.Message,), dict(
  DESCRIPTOR = _DROPSHOTDAMAGEEVENT,
  __module__ = 'api.stats.dropshot_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.DropshotDamageEvent)
  ))
_sym_db.RegisterMessage(DropshotDamageEvent)

DropshotBallPhaseChangeEvent = _reflection.GeneratedProtocolMessageType('DropshotBallPhaseChangeEvent', (_message.Message,), dict(
  DESCRIPTOR = _DROPSHOTBALLPHASECHANGEEVENT,
  __module__ = 'api.stats.dropshot_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.DropshotBallPhaseChangeEvent)
  ))
_sym_db.RegisterMessage(DropshotBallPhaseChangeEvent)

DropshotGameStats = _reflection.GeneratedProtocolMessageType('DropshotGameStats', (_message.Message,), dict(
  DESCRIPTOR = _DROPSHOTGAMESTATS,
  __module__ = 'api.stats.dropshot_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.DropshotGameStats)
  ))
_sym_db.RegisterMessage(DropshotGameStats)


# @@protoc_insertion_point(module_scope)
