# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api/stats/ball_stats.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ...api.stats import stats_pb2
from ...api.stats import extra_mode_stats_pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='api/stats/ball_stats.proto',
  package='api.stats',
  serialized_pb=_b('\n\x1a\x61pi/stats/ball_stats.proto\x12\tapi.stats\x1a\x15\x61pi/stats/stats.proto\x1a api/stats/extra_mode_stats.proto\"\xc6\x01\n\tBallStats\x12\x1f\n\x05speed\x18\x01 \x01(\x0b\x32\x10.api.stats.Speed\x12>\n\x15positional_tendencies\x18\x02 \x01(\x0b\x32\x1f.api.stats.PositionalTendencies\x12%\n\x08\x61verages\x18\x03 \x01(\x0b\x32\x13.api.stats.Averages\x12\x31\n\nextra_mode\x18\x04 \x01(\x0b\x32\x1d.api.stats.ExtraModeBallStats')
  ,
  dependencies=[stats_pb2.DESCRIPTOR,extra_mode_stats_pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_BALLSTATS = _descriptor.Descriptor(
  name='BallStats',
  full_name='api.stats.BallStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='speed', full_name='api.stats.BallStats.speed', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='positional_tendencies', full_name='api.stats.BallStats.positional_tendencies', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='averages', full_name='api.stats.BallStats.averages', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='extra_mode', full_name='api.stats.BallStats.extra_mode', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=99,
  serialized_end=297,
)

_BALLSTATS.fields_by_name['speed'].message_type = stats_pb2._SPEED
_BALLSTATS.fields_by_name['positional_tendencies'].message_type = stats_pb2._POSITIONALTENDENCIES
_BALLSTATS.fields_by_name['averages'].message_type = stats_pb2._AVERAGES
_BALLSTATS.fields_by_name['extra_mode'].message_type = extra_mode_stats_pb2._EXTRAMODEBALLSTATS
DESCRIPTOR.message_types_by_name['BallStats'] = _BALLSTATS

BallStats = _reflection.GeneratedProtocolMessageType('BallStats', (_message.Message,), dict(
  DESCRIPTOR = _BALLSTATS,
  __module__ = 'api.stats.ball_stats_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.BallStats)
  ))
_sym_db.RegisterMessage(BallStats)


# @@protoc_insertion_point(module_scope)
