# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api/stats/goal.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='api/stats/goal.proto',
  package='api.stats',
  serialized_pb=_b('\n\x14\x61pi/stats/goal.proto\x12\tapi.stats\"\x1f\n\tGoalStats\x12\x12\n\ngoal_index\x18\x01 \x01(\x05')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_GOALSTATS = _descriptor.Descriptor(
  name='GoalStats',
  full_name='api.stats.GoalStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='goal_index', full_name='api.stats.GoalStats.goal_index', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=35,
  serialized_end=66,
)

DESCRIPTOR.message_types_by_name['GoalStats'] = _GOALSTATS

GoalStats = _reflection.GeneratedProtocolMessageType('GoalStats', (_message.Message,), dict(
  DESCRIPTOR = _GOALSTATS,
  __module__ = 'api.stats.goal_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.GoalStats)
  ))
_sym_db.RegisterMessage(GoalStats)


# @@protoc_insertion_point(module_scope)
