# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api/stats/extra_mode_stats.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ...api.stats import dropshot_pb2
from ...api.stats import rumble_pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='api/stats/extra_mode_stats.proto',
  package='api.stats',
  serialized_pb=_b('\n api/stats/extra_mode_stats.proto\x12\tapi.stats\x1a\x18\x61pi/stats/dropshot.proto\x1a\x16\x61pi/stats/rumble.proto\"\xc5\x01\n\x11\x45xtraModeGoalInfo\x12\x11\n\tpre_items\x18\x01 \x01(\x08\x12\x18\n\x10scored_with_item\x18\x02 \x01(\x08\x12%\n\tused_item\x18\x03 \x01(\x0e\x32\x12.api.stats.PowerUp\x12.\n\rdropshot_tile\x18\x04 \x01(\x0b\x32\x17.api.stats.DropshotTile\x12\x15\n\rphase_1_tiles\x18\x05 \x01(\x05\x12\x15\n\rphase_2_tiles\x18\x06 \x01(\x05\"U\n\x12\x45xtraModeBallStats\x12?\n\x14\x64ropshot_phase_stats\x18\x01 \x03(\x0b\x32!.api.stats.DropshotBallPhaseStats')
  ,
  dependencies=[dropshot_pb2.DESCRIPTOR,rumble_pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_EXTRAMODEGOALINFO = _descriptor.Descriptor(
  name='ExtraModeGoalInfo',
  full_name='api.stats.ExtraModeGoalInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pre_items', full_name='api.stats.ExtraModeGoalInfo.pre_items', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='scored_with_item', full_name='api.stats.ExtraModeGoalInfo.scored_with_item', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='used_item', full_name='api.stats.ExtraModeGoalInfo.used_item', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dropshot_tile', full_name='api.stats.ExtraModeGoalInfo.dropshot_tile', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='phase_1_tiles', full_name='api.stats.ExtraModeGoalInfo.phase_1_tiles', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='phase_2_tiles', full_name='api.stats.ExtraModeGoalInfo.phase_2_tiles', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=98,
  serialized_end=295,
)


_EXTRAMODEBALLSTATS = _descriptor.Descriptor(
  name='ExtraModeBallStats',
  full_name='api.stats.ExtraModeBallStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dropshot_phase_stats', full_name='api.stats.ExtraModeBallStats.dropshot_phase_stats', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=297,
  serialized_end=382,
)

_EXTRAMODEGOALINFO.fields_by_name['used_item'].enum_type = rumble_pb2._POWERUP
_EXTRAMODEGOALINFO.fields_by_name['dropshot_tile'].message_type = dropshot_pb2._DROPSHOTTILE
_EXTRAMODEBALLSTATS.fields_by_name['dropshot_phase_stats'].message_type = dropshot_pb2._DROPSHOTBALLPHASESTATS
DESCRIPTOR.message_types_by_name['ExtraModeGoalInfo'] = _EXTRAMODEGOALINFO
DESCRIPTOR.message_types_by_name['ExtraModeBallStats'] = _EXTRAMODEBALLSTATS

ExtraModeGoalInfo = _reflection.GeneratedProtocolMessageType('ExtraModeGoalInfo', (_message.Message,), dict(
  DESCRIPTOR = _EXTRAMODEGOALINFO,
  __module__ = 'api.stats.extra_mode_stats_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.ExtraModeGoalInfo)
  ))
_sym_db.RegisterMessage(ExtraModeGoalInfo)

ExtraModeBallStats = _reflection.GeneratedProtocolMessageType('ExtraModeBallStats', (_message.Message,), dict(
  DESCRIPTOR = _EXTRAMODEBALLSTATS,
  __module__ = 'api.stats.extra_mode_stats_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.ExtraModeBallStats)
  ))
_sym_db.RegisterMessage(ExtraModeBallStats)


# @@protoc_insertion_point(module_scope)
