# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api/party.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..api import player_id_pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='api/party.proto',
  package='api',
  serialized_pb=_b('\n\x0f\x61pi/party.proto\x12\x03\x61pi\x1a\x13\x61pi/player_id.proto\"I\n\x05Party\x12 \n\tleader_id\x18\x01 \x01(\x0b\x32\r.api.PlayerId\x12\x1e\n\x07members\x18\x02 \x03(\x0b\x32\r.api.PlayerId')
  ,
  dependencies=[player_id_pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_PARTY = _descriptor.Descriptor(
  name='Party',
  full_name='api.Party',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='leader_id', full_name='api.Party.leader_id', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='members', full_name='api.Party.members', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=45,
  serialized_end=118,
)

_PARTY.fields_by_name['leader_id'].message_type = player_id_pb2._PLAYERID
_PARTY.fields_by_name['members'].message_type = player_id_pb2._PLAYERID
DESCRIPTOR.message_types_by_name['Party'] = _PARTY

Party = _reflection.GeneratedProtocolMessageType('Party', (_message.Message,), dict(
  DESCRIPTOR = _PARTY,
  __module__ = 'api.party_pb2'
  # @@protoc_insertion_point(class_scope:api.Party)
  ))
_sym_db.RegisterMessage(Party)


# @@protoc_insertion_point(module_scope)
